/* (C) 2003 - 2005 by Marcin Wiacek www.mwiacek.com */

#include <stdio.h>
#include <stdarg.h>

#include "../cfg/config.h"
#include "../common/misc/cfg.h"
#include "../common/gsmcomon.h"
#include "../common/misc/misc.h"
#include "../common/gsmstate.h"
#include "../common/service/backup/gsmback.h"
#include "../common/misc/coding/coding.h"

#include "gplus.h"

FILE *DebFile = NULL;
bool UseDeb = false;

void PrintError(GSM_Error error)
{
	if (error != GSM_ERR_NONE) {
		printf("%s",GSM_GetErrorInfo(error));
		exit(-1);
	}
}

#define DCT4GetUEM 50000

GSM_Error DCT4ReplyGetUEM(int MsgLength, unsigned char MsgType, unsigned char *MsgBuffer, void *Struct, int RequestID)
{
	// Struct is pointer to any structure given during frame sending
	char *Buf = (char *)Struct;

	//we must check Type of message
	//and how long it is
	if (MsgType == 0x1B && MsgLength > 4) {
		//message is enough long and now we check subtype
		if (MsgBuffer[3] == 0x08) {
			//if we didn't ask for it return error
			if (RequestID != DCT4GetUEM) return GSM_ERR_FRAME_NOTREQUESTED;
			//we wanted it now and we fill our structure and return
			strcpy(Buf,(const char *)MsgBuffer+10);
			return GSM_ERR_NONE;
		}
	}

	//we must tell Gammu+, that we want to parse this frame by internal
	//module
	return GSM_ERR_FRAME_UNKNOWN;
}

void Identify(int argc, char *argv[])
{
        GSM_Error 		error;
        GSM_StateMachine 	s(DebFile,"",UseDeb);
	unsigned char 		IMEI[20], Model[50], Model2[50], Firm[50], Man[50], Firm2[50];

	printf("Gammu+\n");
	printf("  Version      : %s",VERSION);
	printf(" built %s %s",__TIME__,__DATE__);
	if (strlen(GetCompiler()) != 0) printf(" using %s",GetCompiler());
	printf("\n");
	if (strlen(GetOS()) != 0) printf("  Run on       : %s\n",GetOS());
	
	error = s.OpenFromFile();
	PrintError(error);

	if (!strcmp(s.Protocols->CurrentName,"irdaphonet")) {
		printf("  Connection   : infrared (%s)",s.Protocols->CurrentName);
#ifndef WIN32
		printf(" on %s",s.Devices->CurrentName);
#endif
	} else if (!strcmp(s.Protocols->CurrentName,"bluephonet")) {
		printf("  Connection   : bluetooth (%s)",s.Protocols->CurrentName);
#ifndef WIN32
		printf(" on %s",s.Devices->CurrentName);
#endif
	} else {
		printf("  Connection   : %s",s.Protocols->CurrentName);
	}
	printf("\n  Phone module : %s\n",s.Phones->Current->ModuleName);

	printf("Phone\n");

	error = s.Phones->Current->GetIMEI(IMEI);
	PrintError(error);
	printf("  IMEI         : %s\n",IMEI);

	error = s.Phones->Current->GetManufacturer(Man);
	PrintError(error);
	error = s.Phones->Current->GetModel(Model);
	PrintError(error);
	error = s.Phones->Current->GetCodeNameModel(Model2);
	PrintError(error);
	printf("  Model        : %s %s (%s)\n",Man, Model, Model2);

	error = s.Phones->Current->GetFirmwareVersion(Firm);
	PrintError(error);
	printf("  Firmware     : %s",Firm);
	error = s.Phones->Current->GetFirmwareDate(Firm2);
	PrintError(error);
	printf(" (%s)\n",Firm2);

	if (!strcmp(s.Phones->Current->ModuleName,"dct4 (n6510)")) {
		//we set handler
		s.SetUserReply(DCT4ReplyGetUEM);
		error = s.Phones->Current->Write((unsigned char *)"\x00\x03\x02\x07\x00\x08",6, 0x1B, 4, DCT4GetUEM, Model);
		PrintError(error);
		s.SetUserReply(NULL);		
		printf("  UEM          : %s\n",Model);
	}
}

void Monitor(int argc, char *argv[])
{
        GSM_Error 		error;
        GSM_StateMachine 	s(DebFile,"",UseDeb);
	GSM_PBKStatus 		Status;

	error = s.OpenFromFile();
	PrintError(error);

	while (1) {
		strcpy(Status.Memory,"ME");
		error = s.Phones->Current->GetPBKMemoryStatus(&Status);
		if (error != GSM_ERR_NOTSUPPORTED) PrintError(error);

		strcpy(Status.Memory,"SM");
		error = s.Phones->Current->GetPBKMemoryStatus(&Status);
		if (error != GSM_ERR_NOTSUPPORTED) PrintError(error);
	}
}

void Restore(int argc, char *argv[])
{
        GSM_StateMachine 	s(DebFile,"",UseDeb);
        GSM_Error 		error;
	GSM_Backup		Backup;
	GSM_PBKStatus 		Status;
	GSM_Backup_PBKEntry	*PBKEntry;
	GSM_PBKEntry 		PBKEntry2;
	GSM_Backup_CalEntry	*CalEntry;
	GSM_CalendarEntry	CalEntry2;
	int			i;
	char			ch;

	error = Backup.ReadFromFile(argv[0]);
	PrintError(error);

	error = s.OpenFromFile();
	PrintError(error);

	sprintf(Status.Memory,"ME");
	error = s.Phones->Current->GetPBKMemoryStatus(&Status);
	if (error != GSM_ERR_NONE) {
		PrintError(error);
	} else {
		printf("Do you want to restore phone phonebook ?");
		do {
		      ch = toupper(getc(stdin));
		} while(ch != 'Y' && ch != 'N');
		if (ch == 'Y') {
			sprintf(PBKEntry2.Memory,"ME");
			for(i=1;i<=Status.Used + Status.Free;i++) {
				PBKEntry = NULL;
				while (Backup.GetNext_ME_PBK(&PBKEntry)) {
					if (PBKEntry->GetEntry()->Location != i) continue;
					error = s.Phones->Current->SetPBKMemory(PBKEntry->GetEntry());
					PrintError(error);
					break;
				}
				if (PBKEntry != NULL && PBKEntry->GetEntry()->Location == i) continue;
				PBKEntry2.Location = i;
				error = s.Phones->Current->DeletePBKMemory(&PBKEntry2);
				PrintError(error);
			}
		}
	}

	printf("Do you want to restore phone calendar ?");
	do {
	      ch = toupper(getc(stdin));
	} while(ch != 'Y' && ch != 'N');
	if (ch == 'Y') {
		CalEntry = NULL;
		while (Backup.GetNext_Cal(&CalEntry)) {
			error = s.Phones->Current->AddCalendar(CalEntry->GetEntry());
			PrintError(error);
		}
	}
}

char *DayOfWeekStr(int Year, int Month, int Day)
{
	switch (DayOfWeek(Year,Month,Day)) {
	case 1: return "Monday";
	case 2: return "Tuesday";
	case 3: return "Wednesday";
	case 4: return "Thursday";
	case 5: return "Friday";
	case 6: return "Saturday";
	case 7: return "Sunday";
	default:return "";
	}
}

void GetAllCalendar(int argc, char *argv[])
{
        GSM_StateMachine 	s(DebFile,"",UseDeb);
        GSM_Error 		error;
	GSM_CalendarEntry 	Entry;
	GSM_CalendarSubEntry 	*SubEntry;
	BOOLEAN			start = TRUE;
	int			RepeatEach,RepeatDay,RepeatMonth,RepeatDOW;

	error = s.OpenFromFile();
	PrintError(error);

	while (1) {
		error = s.Phones->Current->GetNextCalendar(&Entry,start);
		if (error == GSM_ERR_EMPTY) break;
		PrintError(error);

		printf("\nNote type    : ");
		switch(Entry.Type) {
		case Calendar_Type_Meeting:
			printf("meeting\n");
			break;
		case Calendar_Type_Memo:
			printf("memo\n");
			break;
		case Calendar_Type_Call:
			printf("phone call\n");
			break;
		case Calendar_Type_Birthday:
			printf("birthday\n");
			break;
		case Calendar_Type_Reminder:
			printf("reminder\n");
			break;
		}

		RepeatEach 	= -1;
		RepeatDay 	= -1;
		RepeatMonth 	= -1;
		RepeatDOW 	= -1;
		SubEntry 	= NULL;
		while (Entry.GetNext(&SubEntry) == TRUE) {
			switch (SubEntry->GetType()) {
			case Calendar_Text_Text:
				printf("Text         : \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case Calendar_Text_Phone:
				printf("Phone number : \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case Calendar_Text_Location:
				printf("Location     : \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case Calendar_DateTime_Start:
				printf("Start time   : %02i:%02i:%02i %02i-%02i-%04i %s\n",
					SubEntry->GetDateTime()->Hour,
					SubEntry->GetDateTime()->Minute,
					SubEntry->GetDateTime()->Second,
					SubEntry->GetDateTime()->Day,
					SubEntry->GetDateTime()->Month,
					SubEntry->GetDateTime()->Year,
					DayOfWeekStr(
						SubEntry->GetDateTime()->Year,
						SubEntry->GetDateTime()->Month,
						SubEntry->GetDateTime()->Day));
				break;
			case Calendar_DateTime_End:
				printf("End time     : %02i:%02i:%02i %02i-%02i-%04i %s\n",
					SubEntry->GetDateTime()->Hour,
					SubEntry->GetDateTime()->Minute,
					SubEntry->GetDateTime()->Second,
					SubEntry->GetDateTime()->Day,
					SubEntry->GetDateTime()->Month,
					SubEntry->GetDateTime()->Year,
					DayOfWeekStr(
						SubEntry->GetDateTime()->Year,
						SubEntry->GetDateTime()->Month,
						SubEntry->GetDateTime()->Day));
				break;
			case Calendar_DateTime_SilentAlarm:
			case Calendar_DateTime_ToneAlarm:
				if (SubEntry->GetType() == Calendar_DateTime_SilentAlarm) {
					printf("Silent alarm : ");
				} else {
					printf("Tone alarm   : ");
				}
				if (Entry.Type == Calendar_Type_Birthday) {
					printf("%02i:%02i:%02i %02i-%02i in each year\n",
						SubEntry->GetDateTime()->Hour,
						SubEntry->GetDateTime()->Minute,
						SubEntry->GetDateTime()->Second,
						SubEntry->GetDateTime()->Day,
						SubEntry->GetDateTime()->Month);
				} else {
					printf("%02i:%02i:%02i %02i-%02i-%04i %s\n",
						SubEntry->GetDateTime()->Hour,
						SubEntry->GetDateTime()->Minute,
						SubEntry->GetDateTime()->Second,
						SubEntry->GetDateTime()->Day,
						SubEntry->GetDateTime()->Month,
						SubEntry->GetDateTime()->Year,
						DayOfWeekStr(
							SubEntry->GetDateTime()->Year,
							SubEntry->GetDateTime()->Month,
							SubEntry->GetDateTime()->Day));
				}
				break;
			case Calendar_DateTime_End_Repeat:
				printf("Repeat end   : %02i:%02i:%02i %02i-%02i-%04i %s\n",
					SubEntry->GetDateTime()->Hour,
					SubEntry->GetDateTime()->Minute,
					SubEntry->GetDateTime()->Second,
					SubEntry->GetDateTime()->Day,
					SubEntry->GetDateTime()->Month,
					SubEntry->GetDateTime()->Year,
					DayOfWeekStr(
						SubEntry->GetDateTime()->Year,
						SubEntry->GetDateTime()->Month,
						SubEntry->GetDateTime()->Day));
				break;
			case Calendar_Int_Repeat_Frequency:
				RepeatEach = SubEntry->GetInt();
				break;
			case Calendar_Int_Repeat_DayOfWeek:
				RepeatDOW = SubEntry->GetInt();
				break;			
			case Calendar_Int_Repeat_Day:
				RepeatDay = SubEntry->GetInt();
				break;
			case Calendar_Int_Repeat_Month:
				RepeatMonth = SubEntry->GetInt();
				break;
			}
		}

		if (RepeatEach != -1 || RepeatDOW != -1 ||
		    RepeatDay != -1 || RepeatMonth != -1) {
			printf("Repeat note  : ");
			if (RepeatEach == 1) printf("each ");
			if (RepeatEach == 2) printf("each second ");
			if (RepeatDOW != -1) {
				switch (RepeatDOW) {
					case 1: printf("Monday ");	break;
					case 2: printf("Tuesday ");	break;
					case 3: printf("Wednesday ");	break;
					case 4: printf("Thursday ");	break;
					case 5: printf("Friday ");	break;
					case 6: printf("Saturday ");	break;
					case 7: printf("Sunday ");	break;
					default:printf(" ");
				}
			}
			if (RepeatDay != -1) {
				printf("%i ",RepeatDay);
				if (RepeatMonth != -1) {
					switch (RepeatMonth) {
						case  1: printf("January ");	break;
						case  2: printf("February ");	break;
						case  3: printf("March ");	break;
						case  4: printf("April ");	break;
						case  5: printf("May ");	break;
						case  6: printf("June ");	break;
						case  7: printf("July ");	break;
						case  8: printf("August ");	break;
						case  9: printf("September ");	break;
						case 10: printf("October ");	break;
						case 11: printf("November ");	break;
						case 12: printf("December ");	break;
						default:printf(" ");
					}
				} else {
					printf("month day ");
				}
			}
			printf("\n");
		}

		start = FALSE;
	}
}

void GetDateTime(int argc, char *argv[])
{
        GSM_StateMachine 	s(DebFile,"",UseDeb);
        GSM_Error 		error;
	GSM_DateTime		DT;

	error = s.OpenFromFile();
	PrintError(error);

	error = s.Phones->Current->GetDateTime(&DT);
	PrintError(error);

	printf("Date & time %02i:%02i:%02i %02i-%02i-%04i %s\n",
		DT.Hour,DT.Minute,DT.Second,DT.Day,DT.Month,DT.Year,
		DayOfWeekStr(DT.Year,DT.Month,DT.Day));
}

void GetMemory(int argc, char *argv[])
{
        GSM_StateMachine 	s(DebFile,"",UseDeb);
        GSM_Error 		error;
	GSM_PBKEntry 		Entry;
	GSM_PBKSubEntry 	*SubEntry;
	int			Start, End, Tmp;

	if (strlen(argv[0]) != 2) return;

	Start = End = atoi(argv[1]);
	if (argc > 2) End = atoi(argv[2]);

	if (End < Start) {
		Tmp   = End;
		End   = Start;
		Start = Tmp;
		printf("Swapped last and first location\n");		
	}

	error = s.OpenFromFile();
	PrintError(error);

	for (Tmp = Start; Tmp <= End; Tmp++) {	
		Entry.ClearAll();

		strcpy(Entry.Memory,argv[0]);
		Entry.Location = Tmp;

		error = s.Phones->Current->GetPBKMemory(&Entry);
		PrintError(error);

		SubEntry = NULL;
		while (Entry.GetNext(&SubEntry) == TRUE) {
			switch (SubEntry->GetType()) {
			case PBK_Text_Phone_General:
				printf("General number \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Phone_Mobile:
				printf("Mobile number \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Phone_Home:
				printf("Home number \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Phone_Work:
				printf("Work number \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Phone_Fax:
				printf("Fax number \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Email:
				printf("Email \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_URL:
				printf("URL \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Postal:
				printf("Postal address \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_UserID:
				printf("User ID \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Note:
				printf("Text note \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_Text_Name:
				printf("Name \"%s\"\n",DecodeUnicodeString(SubEntry->GetText()));
				break;
			case PBK_DateTime_Call:
				printf("Date & time %02i:%02i:%02i %02i-%02i-%04i\n",
					SubEntry->GetDateTime()->Hour,
					SubEntry->GetDateTime()->Minute,
					SubEntry->GetDateTime()->Second,
					SubEntry->GetDateTime()->Day,
					SubEntry->GetDateTime()->Month,
					SubEntry->GetDateTime()->Year);
				break;
			}
		}
	}
}

void ClearAll(int argc, char *argv[])
{
        GSM_StateMachine 	s(DebFile,"",UseDeb);
        GSM_Error 		error;
	GSM_CalendarEntry 	Entry;
	char			ch;

	error = s.OpenFromFile();
	PrintError(error);

	printf("Do you want to clear phone calendar ?");
	do {
	      ch = toupper(getc(stdin));
	} while(ch != 'Y' && ch != 'N');
	if (ch == 'Y') {
		while (1) {
			error = s.Phones->Current->GetNextCalendar(&Entry,TRUE);
			if (error == GSM_ERR_EMPTY) break;
			PrintError(error);
	
			error = s.Phones->Current->DeleteCalendar(&Entry);
			PrintError(error);		
		}
	}
}

void Modules(int argc, char *argv[])
{
	list<GSM_Device_Info>::iterator 	devinfo;
	list<GSM_Protocol_Info>::iterator 	proinfo;
	list<GSM_Phone_Info>::iterator 		phoinfo;
        GSM_Device              		*dev;
        GSM_Protocol            		*pro;
        GSM_Phone               		*pho;
        GSM_StateMachine 			s(DebFile,"",UseDeb);

        printf("Available device modules\n");
        dev = NULL;
        while(1) {
                if (!s.Devices->GetNext(&dev)) break;
		for (devinfo=dev->Info.begin(); devinfo!=dev->Info.end(); ++devinfo) {
                        printf("  \"%s\"\n",devinfo->Device);
		}
        }

        printf("Available protocol modules\n");
        pro = NULL;
        while(1) {
                if (!s.Protocols->GetNext(&pro)) break;
		for (proinfo=pro->Info.begin(); proinfo!=pro->Info.end(); ++proinfo) {
                        printf("  \"%s\" dev \"%s\"\n",proinfo->Protocol,proinfo->Device);
	        }
        }

        printf("Available phone modules\n");
        pho = NULL;
        while(1) {
                if (!s.Phones->GetNext(&pho)) break;
		for (phoinfo=pho->Info.begin(); phoinfo!=pho->Info.end(); ++phoinfo) {
                        printf("  \"%s\" prot \"%s\" cname \"%s\" devname \"%s\" feat \"%s\"\n",
                                phoinfo->Model,phoinfo->Protocol, phoinfo->CodeNameModel, 
                                phoinfo->DeviceModel, phoinfo->Features);
	        }
        }
}

void Help(int argc, char *argv[]);

static CommandLineFunction CommandLineFunctions[] = {
	"--identify",		0, 0, Identify,      "get phone identification", "",
	"--modules",		0, 0, Modules,       "display info about available modules", "",
	"--help",		0, 0, Help,   	     "display this help", "",
	"--monitor",		0, 0, Monitor, 	     "get phone status", "",
	"--getmemory",		2, 3, GetMemory,     "get phonebook entry", "ME|SM location",
	"--restore",		1, 1, Restore,       "restore phone data and settings", "",
	"--getallcalendar",	0, 0, GetAllCalendar,"get all calendar entries", "",
	"--getdatetime",	0, 0, GetDateTime,   "get phone date & time", "",
	"--clearall",		0, 0, ClearAll,	     "", "",
	"",			0, 0, NULL,	     "", "",
};

void Help(int argc, char *argv[])
{
	int i = 0;

	while (CommandLineFunctions[i].Parameter[0] != 0) {
		printf("%s\n  %s %s\n",CommandLineFunctions[i].Help,CommandLineFunctions[i].Parameter,CommandLineFunctions[i].Params);
		i++;
	}
}

int main(int argc, char *argv[])
{
        int i=0,startarg=0;

#ifdef DEBUG
	UseDeb 	= true;
	DebFile = stdout;
#endif
	if (argc > 1 && !strcmp(argv[1],"nothing")) {
		UseDeb 	= false;
		DebFile = NULL;
		startarg++;
	} else if (argc > 1 && !strcmp(argv[1],"textall")) {
		UseDeb 	= true;
		DebFile = stdout;
		startarg++;
	}

	if (argc-startarg==1) {
		Help(argc-2-startarg,argv+2+startarg);
		return 0;
	}

	while(1) {
		if (CommandLineFunctions[i].Parameter[0] == 0) break;
		if (strcmp(argv[1+startarg],CommandLineFunctions[i].Parameter)) {
			i++;
			continue;
		}
		if (argc-2-startarg < CommandLineFunctions[i].MinArguments) {
			printf("Give more arguments\n");
			return 0;
		}
		if (argc-2-startarg > CommandLineFunctions[i].MaxArguments) {
			printf("Too many arguments\n");
			return 0;
		}
		CommandLineFunctions[i].Function(argc-2-startarg,argv+2+startarg);
		return 0;
	}

	Help(argc-2-startarg,argv+2+startarg);
}
